"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CliOutput = void 0;

function _asyncToGenerator2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

  _asyncToGenerator2 = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _workspace() {
  const data = require("@teambit/workspace");

  _workspace = function () {
    return data;
  };

  return data;
}

function _ui() {
  const data = require("@teambit/ui");

  _ui = function () {
    return data;
  };

  return data;
}

function _webpack() {
  const data = require("@teambit/webpack");

  _webpack = function () {
    return data;
  };

  return data;
}

function _bundler() {
  const data = require("@teambit/bundler");

  _bundler = function () {
    return data;
  };

  return data;
}

function _compiler() {
  const data = require("@teambit/compiler");

  _compiler = function () {
    return data;
  };

  return data;
}

function _react() {
  const data = _interopRequireDefault(require("react"));

  _react = function () {
    return data;
  };

  return data;
}

function _ink() {
  const data = require("ink");

  _ink = function () {
    return data;
  };

  return data;
}

function _open() {
  const data = _interopRequireDefault(require("open"));

  _open = function () {
    return data;
  };

  return data;
}

function _outputTemplates() {
  const data = require("./output-templates");

  _outputTemplates = function () {
    return data;
  };

  return data;
}

// Import the IDs & Events
class CliOutput extends _react().default.Component {
  constructor(props) {
    super(props);
    (0, _defineProperty2().default)(this, "eventsListener", event => {
      switch (event.type) {
        case _bundler().ComponentsServerStartedEvent.TYPE:
          this.changeOrAddComponentServer(event.data, event.data.context.id, 'Running');

          if (this.areAllComponentServersRunning()) {
            this.safeOpenBrowser();
          }

          break;

        case _webpack().WebpackCompilationDoneEvent.TYPE:
          this.onWebpackCompilationDone(event);
          break;

        case _ui().UiServerStartedEvent.TYPE:
          this.onUiServerStarted(event);
          break;

        case _workspace().OnComponentChangeEvent.TYPE:
          this.onComponentChange(event);
          break;

        case _workspace().OnComponentAddEvent.TYPE:
          this.onComponentAdd(event);
          break;

        case _workspace().OnComponentRemovedEvent.TYPE:
          this.onComponentRemoved(event);
          break;

        case _compiler().CompilerErrorEvent.TYPE:
          this.setState({
            latestError: event.data.error
          });
          break;

        default:
      }
    });
    (0, _defineProperty2().default)(this, "onWebpackCompilationDone", event => {
      this.setState({
        webpackErrors: [...event.data.stats.compilation.errors],
        webpackWarnings: [...event.data.stats.compilation.warnings]
      });
    });
    (0, _defineProperty2().default)(this, "onComponentRemoved", event => {
      this.onComponentChange(event);
    });
    (0, _defineProperty2().default)(this, "onComponentAdd", event => {
      this.onComponentChange(event);
    });
    this.state = {
      commandFlags: props.commandFlags,
      mainUIServer: null,
      componentServers: [],
      componentChanges: [],
      isBrowserOpen: false,
      latestError: null,
      webpackErrors: [],
      webpackWarnings: []
    };
    this.registerToEvents(props.pubsub);
  }

  registerToEvents(pubsub) {
    pubsub.sub(_ui().UIAspect.id, this.eventsListener);
    pubsub.sub(_webpack().WebpackAspect.id, this.eventsListener);
    pubsub.sub(_bundler().BundlerAspect.id, this.eventsListener);
    pubsub.sub(_workspace().WorkspaceAspect.id, this.eventsListener);
    pubsub.sub(_compiler().CompilerAspect.id, this.eventsListener);
  }

  onUiServerStarted(event) {
    var _this = this;

    return (0, _asyncToGenerator2().default)(function* () {
      const devServers = yield event.data.uiRoot.devServers;
      devServers.forEach(server => {
        _this.changeOrAddComponentServer(server, server.context.id, 'Started');
      });

      _this.setState({
        mainUIServer: event.data
      });

      if (!devServers.length) {
        _this.safeOpenBrowser();
      }
    })();
  }

  onComponentChange(event) {
    this.setState({
      componentChanges: [...this.state.componentChanges, event],
      latestError: null
    });
  }

  // Helpers
  areAllComponentServersRunning() {
    return this.state.componentServers.every(cs => cs.status === 'Running');
  }

  safeOpenBrowser() {
    const {
      suppressBrowserLaunch
    } = this.state.commandFlags;
    const {
      isBrowserOpen,
      mainUIServer
    } = this.state;

    if (mainUIServer && !isBrowserOpen && !suppressBrowserLaunch) {
      this.setState({
        isBrowserOpen: true
      });
      setTimeout(() => (0, _open().default)(`http://${mainUIServer.targetHost}:${mainUIServer.targetPort}/`), 500);
    }
  }

  changeOrAddComponentServer(server, id, status) {
    this.setState({
      componentServers: [...this.state.componentServers.filter(cs => cs.id !== id), {
        server,
        id,
        status
      }]
    });
  }

  render() {
    const {
      componentServers,
      mainUIServer,
      latestError,
      webpackErrors,
      webpackWarnings
    } = this.state;
    const {
      verbose
    } = this.state.commandFlags; // run in scope

    if (mainUIServer && mainUIServer.uiRoot.scope) {
      return /*#__PURE__*/_react().default.createElement(_outputTemplates().UIServersAreReady, {
        mainUIServer: mainUIServer
      });
    }

    return /*#__PURE__*/_react().default.createElement(_react().default.Fragment, null, mainUIServer ? null : /*#__PURE__*/_react().default.createElement(_outputTemplates().Starting, {
      componentServers: componentServers
    }), /*#__PURE__*/_react().default.createElement(_ink().Newline, null), /*#__PURE__*/_react().default.createElement(_outputTemplates().ComponentPreviewServerStarted, {
      items: componentServers
    }), /*#__PURE__*/_react().default.createElement(_ink().Newline, null), /*#__PURE__*/_react().default.createElement(_outputTemplates().TSErrors, {
      latestError: latestError,
      verbose: !!verbose
    }), webpackErrors.map((err, index) => /*#__PURE__*/_react().default.createElement(_ink().Text, {
      key: index
    }, "Error: ", err)), webpackWarnings.map((warning, index) => /*#__PURE__*/_react().default.createElement(_ink().Text, {
      key: index
    }, "Warning: ", warning)), /*#__PURE__*/_react().default.createElement(_outputTemplates().UIServersAreReady, {
      mainUIServer: mainUIServer
    }));
  }

}

exports.CliOutput = CliOutput;

//# sourceMappingURL=start.cmd/cli-output.js.map