"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UIServer = void 0;

function _asyncToGenerator2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

  _asyncToGenerator2 = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _express() {
  const data = _interopRequireDefault(require("express"));

  _express = function () {
    return data;
  };

  return data;
}

function _expressHistoryApiFallback() {
  const data = _interopRequireDefault(require("express-history-api-fallback"));

  _expressHistoryApiFallback = function () {
    return data;
  };

  return data;
}

function _getPort() {
  const data = _interopRequireDefault(require("get-port"));

  _getPort = function () {
    return data;
  };

  return data;
}

function _httpProxy() {
  const data = _interopRequireDefault(require("http-proxy"));

  _httpProxy = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = require("path");

  _path = function () {
    return data;
  };

  return data;
}

function _webpack() {
  const data = _interopRequireDefault(require("webpack"));

  _webpack = function () {
    return data;
  };

  return data;
}

function _webpackDevServer() {
  const data = _interopRequireDefault(require("webpack-dev-server"));

  _webpackDevServer = function () {
    return data;
  };

  return data;
}

function _ui() {
  const data = require("./ui.aspect");

  _ui = function () {
    return data;
  };

  return data;
}

function _webpackDev() {
  const data = require("./webpack/webpack.dev.config");

  _webpackDev = function () {
    return data;
  };

  return data;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2().default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

class UIServer {
  constructor(graphql, expressExtension, ui, uiRoot, uiRootExtension, logger) {
    this.graphql = graphql;
    this.expressExtension = expressExtension;
    this.ui = ui;
    this.uiRoot = uiRoot;
    this.uiRootExtension = uiRootExtension;
    this.logger = logger;
    (0, _defineProperty2().default)(this, "_port", 0);
  }

  getName() {
    return this.uiRoot.name;
  }

  get port() {
    return this._port;
  }
  /**
   * get the webpack configuration of the UI server.
   */


  getDevConfig() {
    var _this = this;

    return (0, _asyncToGenerator2().default)(function* () {
      const aspects = yield _this.uiRoot.resolveAspects(_ui().UIRuntime.name);
      const aspectsPaths = aspects.map(aspect => aspect.aspectPath);
      return (0, _webpackDev().devConfig)(_this.uiRoot.path, [yield _this.ui.generateRoot(aspects, _this.uiRootExtension)], _this.uiRoot.name, aspectsPaths);
    })();
  }
  /**
   * start a UI server.
   */


  start({
    port
  } = {}) {
    var _this2 = this;

    return (0, _asyncToGenerator2().default)(function* () {
      const app = _this2.expressExtension.createApp(); // TODO: better handle ports.


      const selectedPort = yield _this2.selectPort(port || 4000);
      const root = (0, _path().join)(_this2.uiRoot.path, '/public');
      const server = yield _this2.graphql.createServer({
        app
      });
      yield _this2.configureProxy(app, server);
      app.use(_express().default.static(root));
      app.use((0, _expressHistoryApiFallback().default)('index.html', {
        root
      }));
      server.listen(selectedPort);
      _this2._port = selectedPort;

      _this2.logger.info(`UI server of ${_this2.uiRootExtension} is listening to port ${selectedPort}`);
    })();
  } // TODO - check if this is necessary


  configureProxy(app, server) {
    var _this3 = this;

    return (0, _asyncToGenerator2().default)(function* () {
      const proxServer = _httpProxy().default.createProxyServer();

      proxServer.on('error', e => _this3.logger.error(e.message));
      const proxyEntries = _this3.uiRoot.getProxy ? yield _this3.uiRoot.getProxy() : [];
      server.on('upgrade', function (req, socket, head) {
        const entry = proxyEntries.find(proxy => proxy.context.some(item => item === req.url));
        if (!entry) return;
        proxServer.ws(req, socket, head, {
          target: entry.target
        });
      });
      proxyEntries.forEach(entry => {
        entry.context.forEach(route => {
          app.use(`${route}/*`, (req, res) => {
            proxServer.web(req, res, _objectSpread(_objectSpread({}, entry), {}, {
              target: `${entry.target}/${req.originalUrl}`
            }));
          });
        });
      });
    })();
  }
  /**
   * start a UI dev server.
   */


  dev({
    port
  } = {}) {
    var _this4 = this;

    return (0, _asyncToGenerator2().default)(function* () {
      const selectedPort = yield _this4.selectPort(port); // improve port management.

      yield _this4.start({
        port: yield (0, _getPort().default)({
          port: 4000
        })
      });
      const config = yield _this4.getDevConfig();
      const compiler = (0, _webpack().default)(config);
      const devServerConfig = yield _this4.getDevServerConfig(config.devServer);
      const devServer = new (_webpackDevServer().default)(compiler, devServerConfig);
      devServer.listen(selectedPort);
      return devServer;
    })();
  }

  selectPort(port) {
    return (0, _asyncToGenerator2().default)(function* () {
      if (port) return port;
      return (0, _getPort().default)({
        port: _getPort().default.makeRange(3100, 3200)
      });
    })();
  }

  getProxy() {
    var _this5 = this;

    return (0, _asyncToGenerator2().default)(function* () {
      var _this5$uiRoot$getProx, _this5$uiRoot;

      const proxyEntries = (yield (_this5$uiRoot$getProx = (_this5$uiRoot = _this5.uiRoot).getProxy) === null || _this5$uiRoot$getProx === void 0 ? void 0 : _this5$uiRoot$getProx.call(_this5$uiRoot)) || [];
      const gqlProxies = [{
        context: ['/graphql', '/api'],
        target: 'http://localhost:4000',
        changeOrigin: true
      }, {
        context: ['/subscriptions'],
        target: 'ws://localhost:4000',
        ws: true
      }];
      return gqlProxies.concat(proxyEntries);
    })();
  }

  getDevServerConfig(config) {
    var _this6 = this;

    return (0, _asyncToGenerator2().default)(function* () {
      const proxy = yield _this6.getProxy();

      const devServerConf = _objectSpread(_objectSpread({}, config), {}, {
        proxy
      });

      return devServerConf;
    })();
  }

  static create(props) {
    return new UIServer(props.graphql, props.express, props.ui, props.uiRoot, props.uiRootExtension, props.logger);
  }

}

exports.UIServer = UIServer;

//# sourceMappingURL=ui-server.js.map